"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'no-await-sync-events';
const USER_EVENT_ASYNC_EXCEPTIONS = ['type', 'keyboard'];
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow unnecessary `await` for sync events',
            category: 'Best Practices',
            recommended: 'error',
        },
        messages: {
            noAwaitSyncEvents: '`{{ name }}` is sync and does not need `await` operator',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context, _, helpers) {
        return {
            'AwaitExpression > CallExpression'(node) {
                const simulateEventFunctionIdentifier = node_utils_1.getDeepestIdentifierNode(node);
                const isSimulateEventMethod = helpers.isUserEventMethod(simulateEventFunctionIdentifier) ||
                    helpers.isFireEventMethod(simulateEventFunctionIdentifier);
                if (!isSimulateEventMethod) {
                    return;
                }
                const lastArg = node.arguments[node.arguments.length - 1];
                const hasDelay = node_utils_1.isObjectExpression(lastArg) &&
                    lastArg.properties.some((property) => node_utils_1.isProperty(property) &&
                        experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                        property.key.name === 'delay' &&
                        node_utils_1.isLiteral(property.value) &&
                        property.value.value > 0);
                const simulateEventFunctionName = simulateEventFunctionIdentifier.name;
                if (USER_EVENT_ASYNC_EXCEPTIONS.includes(simulateEventFunctionName) &&
                    hasDelay) {
                    return;
                }
                context.report({
                    node,
                    messageId: 'noAwaitSyncEvents',
                    data: {
                        name: `${node_utils_1.getPropertyIdentifierNode(node).name}.${simulateEventFunctionName}`,
                    },
                });
            },
        };
    },
});
